# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2025-2026, by Samuel Williams.

# @namespace
module Async
	# @namespace
	module Service
		# @namespace
		module Supervisor
			VERSION = "0.17.0"
		end
	end
end

